@@ -139,7 +139,7 @@ class BaseTagger(BaseParser):
                 transform_kwargs=word_transform_kwargs,
                 next_emb_params=word_next_emb_params, embs=self.embs,
                 loglevel=0 if log_file is None else
-                         1 if log_file == sys.stdin else
+                         1 if log_file == sys.stdout else
                          2
             )
             ds.add('x', x)
@@ -187,7 +187,7 @@ class BaseTagger(BaseParser):
                 if not WordEmbeddings.transform(
                     ds_, sentences, batch_size=batch_size,
                     loglevel=0 if log_file is None else
-                             1 if log_file == sys.stdin else
+                             1 if log_file == sys.stdout else
                              2
                 ):
                     ds_.transform(sentences)
@@ -202,7 +202,7 @@ class BaseTagger(BaseParser):
             ds = self._ds
         res = []
         for name in self._ds.list():
-            ds_ = self._ds.get_dataset(name)
+            ds_, collate_kwargs = self._ds.get(name)
             name_ = name.split('_', maxsplit=1)
             typ, idx = name_[0], name_[1] if len(name_) > 1 else None
             if typ == 'x':
@@ -213,18 +213,18 @@ class BaseTagger(BaseParser):
                 if not res_:
                     res_ = ds_.transform_collate(
                         sentences, batch_size=batch_size,
-                        collate_kwargs={'with_lens': False}
+                        collate_kwargs=collate_kwargs
                     )
                 res.append(res_)
             elif typ == 't':
                 res.append(ds_.transform_collate(
                     tags[int(idx)], batch_size=batch_size,
-                    collate_kwargs={'with_lens': False}
+                    collate_kwargs=collate_kwargs
                 ))
             elif labels and typ == 'y':
                 res.append(ds_.transform_collate(
                     labels, batch_size=batch_size,
-                    collate_kwargs={'with_lens': False}
+                    collate_kwargs=collate_kwargs
                 ))
         for batch in zip(*res):
             batch_ = []
@@ -288,7 +288,7 @@ class BaseTagger(BaseParser):
         Any other changes most likely won't allow the tagger to load.
         """
         assert self._ds is not None, \
-               "ERROR: The tagger doesn't have a dataset to save"
+            "ERROR: The tagger doesn't have a dataset to save"
         assert self._model, "ERROR: The tagger doesn't have a model to save"
         self._save_dataset(name)
         model_config_fn, model_fn, _, _, cdict_fn = \
